'use strict';

angular.module('adf.widget.electre.lavoisier')
    .service('domService',
        [
            function () {
                /**
                 * The dom element to use.
                 * @type domElement_|Element
                 */
                var domElement;

                return {
                    /**
                     * Give the 'domElement' the value of 'domElement_'.
                     * @param {Element} domElement_
                     * @returns {dom_L6.domAnonym$1}
                     */
                    setDom : function (domElement_) {
                        domElement = domElement_;
                        return this;
                    },
                    /**
                     * Give the 'domElement' object.
                     * @returns {domElement_|Element}
                     */
                    getDom : function () {
                        return domElement;
                    },
                    /**
                     * Create a HTML DOM Node with the specified 'tagName'.
                     * @param {String} tagName
                     * @returns {dom_L6.domAnonym$1}
                     */
                    createElement : function (tagName) {
                        domElement = document.createElement(tagName);
                        return this;
                    },
                    /**
                     * Donne à l'attribut 'attributeName' la valeur 'attributeValue'.
                     * @param {String} attributeName
                     * @param {String} attributeValue
                     * @returns {dom_L6.domAnonym$1}
                     */
                    setAttribute : function (attributeName, attributeValue) {
                        domElement.setAttribute(attributeName, attributeValue);
                        return this;
                    },
                    /**
                     * Donne la valeur de l'attribut demandé.
                     * @param {String} attributeName
                     * @returns {String}
                     */
                    getAttribute : function (attributeName) {
                        return domElement.getAttribute(attributeName);
                    },
                    /**
                     * Ajoute un Node enfant avant la fin de celui-ci.
                     * @param {Element|Array<Element>} childNode
                     * @returns {dom_L6.domAnonym$1}
                     */
                    appendChild : function (childNode) {
                        domElement.appendChild(childNode);
                        return this;
                    }
                };
            }
        ]
    ).service('lavoisierHttpRequestService',
         [
             "$http", "$q",
             function ($http, $q) {
                 /**
                  * Used as valid parameters for the http request.
                  * @type Object
                  */
                 var validParams = {};
                 /**
                  *
                  * @param {Object} params Containing information about the url to pass to the $http service.
                  */
                 function recupUrl (params) {
                     // Détection du '?' dans l'url
                     if (params.url.match(/\u003f/) && params.params) {
                         // Récupération distincte de l'url avant et après le ?
                         validParams.url = params.url.substring(0, params.url.search(/\u003f/));
                         validParams.params = {};

                         // Récupération de la chaine de paramètre.
                         var parametres = params.url.substring(params.url.search(/\u003f/) + 1);

                         // Si la chaine de paramètre contient un '&'
                         if (parametres.indexOf("&") !== -1) {
                             parametres = parametres.split("&");
                             for (var i = 0 ; i < parametres.length ; i++) parametres[i] = parametres[i].split("=");
                         } else {
                             parametres = [parametres.split("=")];
                         }

                         // Récupération de la liste des paramètres.
                         for (var i = 0 ; i < parametres.length ; i++) validParams.params[parametres[i][0]] = parametres[i][1];

                         for (var index in params.params) validParams.params[index] = params.params[index];
                     } else {
                         validParams.url = params.url;
                         validParams.params = params.params;
                     }
                 }

                 return {
                     /**
                      * Call to $http service with validate parameters.
                      * @param {Object} params
                      * @returns {$q@call;defer.promise}
                      */
                     getResult : function (params) {
                         var deferred = $q.defer();
                         recupUrl(params);

                         $http(
                             {
                                 url : validParams.url,
                                 params : (validParams.params ? validParams.params : {})
                             }
                         ).success(
                             function (data) {
                                 deferred.resolve(data);
                             }
                         ).error(
                             function (data) {
                                 deferred.reject();
                             }
                         );

                         return deferred.promise;
                     },
                     /**
                      * Call to $http service without validated parameters.
                      * @param {Object} params
                      * @returns {$q@call;defer.promise}
                      */
                     getResult_2 : function (params) {
                         var deferred = $q.defer();

                         $http(
                             {
                                 url : params.url,
                                 params : (params.params ? params.params : {})
                             }
                         ).success(
                             function (data) {
                                 deferred.resolve(data);
                             }
                         ).error(
                             function (data) {
                                 deferred.reject();
                             }
                         );

                         return deferred.promise;
                     }
                 };
             }
         ]
    ).service('parserService',
         function () {
             /**
              * Parser XML et JSON.
              * @type X2JS
              */
             var x2js = new X2JS();
             /**
              * Parser DOM (HTML, XML, ...).
              * @type DOMParser
              */
             var domParser = new DOMParser();

             return {
                 /**
                  * Parse a string xml into a DOM XML using the 'domParser'.
                  * @param {String} xmlString XML specified as string
                  * @returns {Element} DOM XML
                  */
                 xmlDomFromString : function (xmlString) {
                     return domParser.parseFromString(xmlString, 'text/xml');
                 },
                 /**
                  * Parse a string html into a DOM HTML using the 'domParser'.
                  * @param {String} htmlString HTML specified as string
                  * @returns {Element} DOM HTML
                  */
                 htmlDomFromString : function (htmlString) {
                     return domParser.parseFromString(htmlString, 'text/html');
                 },
                 /**
                  * Parse a XML specified as string to a DOM Object.
                  * @param {String} xmlString XML specified as string
                  * @returns {Element} DOM XML
                  */
                 parseXmlString : function (xmlString) {
                     return x2js.parseXmlString(xmlString);
                 },
                 /**
                  * Convert a specified JSON Object to XML DOM Object.
                  * @param {Object} jsonObject
                  * @returns {Element} DOM XML
                  */
                 json2xml : function (jsonObject) {
                     return x2js.json2xml(jsonObject);
                 },
                 /**
                  * Convert a specified XML DOM Object to JSON Object.
                  * @param {Element} xmlDom
                  * @returns {Object} JSON Object
                  */
                 xml2json : function (xmlDom) {
                     return x2js.xml2json(xmlDom);
                 },
                 /**
                  * Convert a specified JSON Object to XML String.
                  * @param {Object} jsonObject
                  * @returns {String} XML String
                  */
                 json2xml_str : function (jsonObject) {
                     return x2js.json2xml_str(jsonObject);
                 },
                 /**
                  * Convert a specified XML String to JSON Object.
                  * @param {String} xmlString
                  * @returns {Object}
                  */
                 xml_str2json : function (xmlString) {
                     return x2js.xml_str2json(xmlString);
                 }
             };
         }
    );